"""
Copyright: Pierre Dubertrand
Author: Pierre Dubertrand

Name-en-US: Batch Load HDRIs...
Description-en-US: Load multiple HDRI files at once


"""

import c4d
from pathlib import Path


HDRI_LOADER_CRN = 1062504

LOAD_MODE_BASECONTAINER = 2000
LOAD_MODE = 2001

MODE_OBJ = 0
MODE_MTL = 1
MODE_SELECT = 2

BTN_SAVE = 9001
BTN_CANCEL = 9002

CORONA_SELECT_SHADER_ROW_TEXTURE = 24101
CORONA_SELECT_SHADER_VALUE_ID_STEP = 10


doc = c4d.documents.GetActiveDocument()


class HDRILoaderDialog(c4d.gui.GeDialog):

    def __init__(self):

        self.userCancel = False
        self.userOK = False

    def InitValues(self):

        # Create the base container to store the
        # preferences if it doesnt exist yet
        docBC = doc.GetDataInstance()

        if not docBC.GetContainerInstance(HDRI_LOADER_CRN):
            bc = c4d.BaseContainer()
            bc.SetId(LOAD_MODE_BASECONTAINER)
            bc[LOAD_MODE] = MODE_OBJ
            docBC.SetContainer(HDRI_LOADER_CRN, bc)

        # get stored value and set it as the active mode
        bc = docBC.GetContainer(HDRI_LOADER_CRN)

        default_mode = bc[LOAD_MODE]

        self.SetInt32(LOAD_MODE, default_mode)

        return True

    def CreateLayout(self):

        self.SetTitle("Corona HDRI Loader")

        # group layout
        self.GroupBegin(id=1000, flags=c4d.BFH_CENTER | c4d.BFV_CENTER, cols=2,
                        rows=1, title="Settings",
                        groupflags=c4d.BFV_GRIDGROUP_EQUALROWS)

        # group settings
        self.GroupBorder(c4d.BORDER_WITH_TITLE_BOLD)
        self.GroupBorderSpace(5, 5, 5, 5)

        # group items
        self.AddStaticText(
            1001, c4d.BFH_FIT | c4d.BFV_FIT, name="Files Loading Mode")
        self.AddComboBox(LOAD_MODE, c4d.BFH_FIT | c4d.BFV_FIT)

        # dropdown menu items
        self.AddChild(LOAD_MODE, MODE_OBJ, "Corona Sky Objects")
        self.AddChild(LOAD_MODE, MODE_MTL, "Corona Light Materials")
        self.AddChild(LOAD_MODE, MODE_SELECT, "Corona Sky + Select Shader")

        self.GroupEnd()

        self.AddDlgGroup(c4d.DLG_OK | c4d.DLG_CANCEL)

        return True

    def Command(self, id, msg):

        if id == c4d.DLG_OK:
            self.userOK = True
            bc = doc.GetDataInstance().GetContainer(
                HDRI_LOADER_CRN)    # Get the plugin BaseContainer
            # Set the new value from user choice
            bc[LOAD_MODE] = self.GetInt32(LOAD_MODE)
            doc.GetDataInstance().SetContainer(HDRI_LOADER_CRN, bc)
            self.Close()

        elif id == c4d.DLG_CANCEL:
            self.userCancel = True
            self.Close()

        return True


def create_bitmap_shader(doc, filepath: str) -> c4d.BaseShader:
    shader = c4d.BaseShader(c4d.Xbitmap)
    shader[c4d.BITMAPSHADER_FILENAME] = filepath
    return shader


def create_selectshader(doc, files: list) -> c4d.BaseShader:
    sel_shader = c4d.BaseShader(1052988)
    sel_shader[c4d.CORONA_SELECT_SHADER_ROW_COUNT] = len(files)
    for i, v in enumerate(files):
        filepath = str(v)
        shader = create_bitmap_shader(doc, filepath)
        shader.InsertUnder(sel_shader)
        doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, shader)
        # id = base_id if i == 0 else base_id + CORONA_SELECT_SHADER_VALUE_ID_STEP * i
        if i == 0:
            id = CORONA_SELECT_SHADER_ROW_TEXTURE
        else:
            id = CORONA_SELECT_SHADER_ROW_TEXTURE + CORONA_SELECT_SHADER_VALUE_ID_STEP * i
        sel_shader[id] = shader
    return sel_shader


def create_light_mtl(doc, file: Path) -> c4d.BaseMaterial:
    filepath = str(file.resolve())
    shader = create_bitmap_shader(doc, filepath)
    lmtl = c4d.BaseMaterial(1035372)
    lmtl[c4d.ID_BASELIST_NAME] = file.stem
    lmtl[c4d.CORONA_LIGHT_MATERIAL_EMISSION_TEXTURE] = shader
    lmtl.InsertShader(shader)
    doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, shader)
    doc.InsertMaterial(lmtl)
    doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, lmtl)
    return lmtl


def create_csky(doc, shader: c4d.BaseShader, name: str = None) -> c4d.BaseObject:
    corona_sky = c4d.BaseObject(1053478)
    corona_sky[c4d.CORONA_SKYOBJECT_TYPE] = 2
    corona_sky[c4d.CORONA_SKYOBJECT_SHADER] = shader
    if name:
        corona_sky[c4d.ID_BASELIST_NAME] = name
    corona_sky.InsertShader(shader)
    doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, shader)
    doc.InsertObject(corona_sky)
    doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, corona_sky)
    return corona_sky


def batch_create_csky(doc, file: Path) -> c4d.BaseObject:
    filepath = str(file.resolve())
    shader = create_bitmap_shader(doc, filepath)
    corona_sky = c4d.BaseObject(1053478)
    corona_sky[c4d.CORONA_SKYOBJECT_TYPE] = 2
    corona_sky[c4d.CORONA_SKYOBJECT_SHADER] = shader
    # set the sky object name from the hdri name
    corona_sky[c4d.ID_BASELIST_NAME] = file.stem
    corona_sky.InsertShader(shader)
    doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, shader)
    doc.InsertObject(corona_sky)
    doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, corona_sky)
    return corona_sky


def main():

    DEF_PATH = ''

    if c4d.GetClipboardType() == c4d.CLIPBOARDTYPE_STRING and c4d.GetStringFromClipboard():
        DEF_PATH = Path(c4d.GetStringFromClipboard())
        if DEF_PATH.exists():
            DEF_PATH = DEF_PATH
        else:
            DEF_PATH = ''

    # Open the folder containing the files
    folder = c4d.storage.LoadDialog(type=c4d.FILESELECTTYPE_ANYTHING, title='Select a folder',
                                    flags=c4d.FILESELECT_DIRECTORY, force_suffix='', def_path=DEF_PATH, def_file='')

    try:
        folder = Path(folder)
    except TypeError:
        print("[Batch Load HDRIs] No selection was made : operation aborted")
        return

    extensions = (".hdr", ".exr")

    doc.StartUndo()

    if folder.exists():
        files = []
        for f in folder.iterdir():
            if f.suffix in extensions:
                files.append(f)

    if files:
        # Open the settings
        dlg = HDRILoaderDialog()
        dlg.Open(dlgtype=c4d.DLG_TYPE_MODAL, xpos=-2, ypos=-2)
        if dlg.userCancel == True:
            print("[Batch Load HDRIs] Operation canceled by the user")
            return
        elif dlg.userOK == True:
            # Get the user choice
            bc = doc.GetDataInstance().GetContainer(HDRI_LOADER_CRN)
            mode = bc[LOAD_MODE]
            # Process the files according to the selected mode
            if mode == MODE_OBJ:
                [batch_create_csky(doc, file) for file in files]
            elif mode == MODE_MTL:
                [create_light_mtl(doc, file) for file in files]
            elif mode == MODE_SELECT:
                sel_shader = create_selectshader(doc, files)
                create_csky(doc, sel_shader, "HDRIs")
        else:
            print("[Batch Load HDRIs] Window closed : operation aborted")
            return

    doc.EndUndo()

    c4d.EventAdd()


if __name__ == '__main__':
    main()
