"""
Copyright: Pierre Dubertrand
Author: Pierre Dubertrand

Name-en-US: Convert To C4D Bitmap Shaders
Description-en-US: Convert all the Corona Bitmap shaders in the selected materials to Cinema 4D Bitmap shaders.


"""

import c4d
import time

doc = c4d.documents.GetActiveDocument()

def iter_shaders(material: c4d.BaseMaterial, mask: int = None) -> list:

    shaders = []

    shader = material.GetFirstShader()

    if shader == None:
        return

    while shader:
        # Visiting the node
        if mask:
            if shader.GetType() == mask:
                shaders.append(shader)
        else:
            shaders.append(shader)
        # progress the reference
        if shader.GetDown():
            shader = shader.GetDown()
        elif shader.GetNext():
            shader = shader.GetNext()
        else:
            while shader.GetUp():
                if shader.GetUp().GetNext():
                    shader = shader.GetUp().GetNext()
                    break
                shader = shader.GetUp()
            else:
                shader = None

    return shaders


def get_layershader_layer(doc: c4d.documents.BaseDocument, layer_shader: c4d.BaseShader, target_shader: c4d.BaseShader = None) -> c4d.LayerShaderLayer:
    if layer_shader.GetType() == c4d.Xlayer:
        layer = layer_shader.GetFirstLayer()
        while layer != None:
            if layer.GetType() == c4d.TypeShader:
                shader = layer.GetParameter(c4d.LAYER_S_PARAM_SHADER_LINK)
                if shader == target_shader:
                    return layer
                layer = layer.GetNext()
            elif layer.GetNext():
                layer = layer.GetNext()
            else:
                layer = None


def convert_cr_bitmap(doc, corona_bitmap, remove=True):
    c4d_bitmap = create_bitmap_shader(
        corona_bitmap[c4d.CORONA_BITMAP_FILENAME], corona_bitmap[c4d.CORONA_BITMAP_COLORPROFILE], corona_bitmap[c4d.ID_BASELIST_NAME], corona_bitmap[c4d.CORONA_BITMAP_EXPOSURE], corona_bitmap)
    c4d_bitmap_added = True
    if corona_bitmap.GetUp():
        parent = corona_bitmap.GetUp()
        c4d_bitmap.InsertUnder(parent)  
        doc.AddUndo(c4d.UNDOTYPE_NEW, c4d_bitmap)
        if parent.CheckType(c4d.Xlayer):
            # Looking for the shader link in the Layer Shader Layer
            layer = get_layershader_layer(doc, parent, corona_bitmap)
            c4d_bitmap[c4d.ID_BASELIST_NAME] = layer.GetName(doc)
            doc.AddUndo(c4d.UNDOTYPE_CHANGE_SMALL, parent)
            layer.SetParameter(c4d.LAYER_S_PARAM_SHADER_LINK, c4d_bitmap)
        else:
            parent_shader_slot = get_bc_id(
                parent.GetDataInstance(), corona_bitmap)
            doc.AddUndo(c4d.UNDOTYPE_CHANGE, parent)
            parent[parent_shader_slot] = c4d_bitmap

    elif corona_bitmap.GetMain():
        parent = corona_bitmap.GetMain()
        doc.AddUndo(c4d.UNDOTYPE_CHANGE, parent)
        parent.InsertShader(c4d_bitmap)
        doc.AddUndo(c4d.UNDOTYPE_NEW, c4d_bitmap)
        parent_shader_slot = get_bc_id(parent.GetDataInstance(), corona_bitmap)       
        parent[parent_shader_slot] = c4d_bitmap
    else:
        c4d_bitmap_added = False

    if remove and c4d_bitmap_added:
        doc.AddUndo(c4d.UNDOTYPE_DELETEOBJ, corona_bitmap)
        corona_bitmap.Remove()
    return c4d_bitmap


def create_bitmap_shader(filepath, colorspc, basename, exposure, shader):
    bitmap = c4d.BaseShader(c4d.Xbitmap)
    bitmap[c4d.BITMAPSHADER_FILENAME] = filepath
    bitmap[c4d.BITMAPSHADER_COLORPROFILE] = colorspc
    bitmap[c4d.ID_BASELIST_NAME] = basename
    bitmap[c4d.BITMAPSHADER_EXPOSURE] = exposure if colorspc in [1, 3] else exposure / 2.2
    return bitmap  # Return the bitmap shader object for further material insertion


def get_bc_id(target_bc, target_value):
    for index, value in target_bc:
        if value == target_value:
            return index


def main():

    materials = doc.GetActiveMaterials()

    doc.StartUndo()

    # Performance measurements
    tic = time.perf_counter()
    c = 0

    # Main loop
    for material in materials:

        # Step over non Corona Physical materials or Corona Legacy materials.
        if material.GetType() not in (1056306, 1032100):
            continue

        for bitmap_shader in iter_shaders(material, 1036473):
            c += 1
            convert_cr_bitmap(doc, bitmap_shader)

    # Performance measurements
    toc = time.perf_counter()
    print(
        f"[Corona Bitmap tp C4D Bitmap]: {c} shaders converted in {toc - tic:0.4f} seconds")

    doc.EndUndo()

    c4d.EventAdd()


if __name__ == '__main__':
    main()